/* DS1820 test program that reads temperature form DS1820. */

#include <pic.h>

#define XTAL_FREQ	4MHZ
#define BITNUM(adr, bit)       ((unsigned)(&adr)*8+(bit))

static bit D_PIN @ BITNUM(PORTB,1);
static bit D_TRIS @ BITNUM(TRISB,1);
static bit LED_PIN @ BITNUM(PORTB,2);
static bit LED_TRIS @ BITNUM(TRISB,2);
static bit A_7SEG      @ BITNUM(PORTD,7);
static bit A_7SEG_TRIS @ BITNUM(TRISD,7);
static bit B_7SEG      @ BITNUM(PORTD,6);
static bit B_7SEG_TRIS @ BITNUM(TRISD,6);
static bit C_7SEG      @ BITNUM(PORTD,5);
static bit C_7SEG_TRIS @ BITNUM(TRISD,5);
static bit D_7SEG      @ BITNUM(PORTD,4);
static bit D_7SEG_TRIS @ BITNUM(TRISD,4);
static bit E_7SEG      @ BITNUM(PORTD,3);
static bit E_7SEG_TRIS @ BITNUM(TRISD,3);
static bit F_7SEG      @ BITNUM(PORTD,2);
static bit F_7SEG_TRIS @ BITNUM(TRISD,2);
static bit G_7SEG      @ BITNUM(PORTD,1);
static bit G_7SEG_TRIS @ BITNUM(TRISD,1);
static bit H_7SEG      @ BITNUM(PORTD,0);
static bit H_7SEG_TRIS @ BITNUM(TRISD,0);
static bit HUNDREDS_7SEG      @ BITNUM(PORTA,0);
static bit HUNDREDS_7SEG_TRIS @ BITNUM(TRISA,0);
static bit TENS_7SEG      @ BITNUM(PORTA,1);
static bit TENS_7SEG_TRIS @ BITNUM(TRISA,1);
static bit ONES_7SEG      @ BITNUM(PORTA,2);
static bit ONES_7SEG_TRIS @ BITNUM(TRISA,2);
static bit POINT_7SEG      @ BITNUM(PORTA,3);
static bit POINT_7SEG_TRIS @ BITNUM(TRISA,3);

#define LED7_SEGPORT	PORTD
#define LED7_SEGDIR	TRISD
#define COMMON_SEGPORT	PORTA
#define COMMON_SEGDIR	TRISA


#include "delay.h"
#include "delay.c"

#include "1wire.c"
//************ DEFINE CHARACTERS *****************
                            //          A
#define LA   (-0x80)        //        ****
#define LB   (-0x40)        //     F *    *  B
#define LC   (-0x20)        //       *  G *
#define LD   (-0x10)        //        ****
#define LE   (-0x08)        //     E *    *  C
#define LF   (-0x04)        //       *    *
#define LG   (-0x02)        //        ****   * DOT
#define LDOT (-0x01)        //          D
#define LBLANK (0xFF)


#define NUM0  (LBLANK+LA+LB+LC+LD+LE+LF)    //-- '0'
#define NUM1  (LBLANK+LB+LC)                //-- '1'
#define NUM2  (LBLANK+LA+LB+LG+LE+LD)       //-- '2'
#define NUM3  (LBLANK+LA+LB+LC+LD+LG)       //-- '3'
#define NUM4  (LBLANK+LF+LG+LB+LC)          //-- '4'
#define NUM5  (LBLANK+LA+LF+LG+LC+LD)       //-- '5'
#define NUM6  (LBLANK+LA+LF+LE+LD+LC+LG)    //-- '6'
#define NUM7  (LBLANK+LA+LB+LC)             //-- '7'
#define NUM8  (LBLANK+LA+LB+LC+LD+LE+LF+LG) //-- '8'
#define NUM9  (LBLANK+LA+LB+LC+LF+LG)       //-- '9'

#define MINUS -(LG)+1                   //-- '-'      - Negative
#define THIGH LB+LC+LE+LF+LG       //-- 'H'      - High Level
#define TLOW  LF+LE+LD             //-- 'L'      - Low Level
#define PROGP LF+LE+LG+LB+LA       //-- 'P'      - Program
#define PROGR LG+LE                //-- 'r'      - r
#define PROGO LG+LC+LD+LE          //-- 'o'      - o
#define PROGG LA+LB+LC+LD+LF+LG    //-- 'g'      - g
#define CLRC  LG+LE+LD             //-- 'c'      -Clear
#define CLRL  LF+LE                //-- 'l'
#define CLRR  LE+LG                //-- 'r'
#define ERROR LA+LF+LG+LE+LD       //-- 'E'      - Error
#define PRGDONE LA+LG+LD           //-- 3 bars one above the other
#define L_c   CLRC                 //-- 'c'
#define L_A   LE+LF+LA+LB+LC+LG    //-- 'A'
#define L_t   LF+LE+LD+LG          //-- 't'


#define DOT  LDOT     //-- '.' The Dot Shows current Position for altering
const char Digits[]={NUM0,NUM1,NUM2,NUM3,NUM4,NUM5,NUM6,NUM7,NUM8,NUM9};

#define T_NEGATIVE  2     //-- Indicates a Negative Value


//************** VARIABLES **************
unsigned char  D_Mag,D_Sign,Refresh_Count;
struct
 {
  char Point;
  char Ones;               //-- Editable Areas
  char Tens;
  char Hundreds;
  char Type;               //-- Type of area - High/Low/Clear/Program
 }Area,SevenSeg;


//************ Functions *************
void ReadTemp(void);		//-- Read current temperature from DS1820
void ConvertForDisplay(void);
void Refresh_7Segment(void);
//************************************************************************
//                             main
//************************************************************************
void main(void)
{
	GIE = 0;		// Disble all interrupt
/* initialize DS1820 wire pin (PORTB.1) as output */
	D_TRIS=1;		//-- Set direction as OUTPUT
	D_PIN=1;		//-- Set to HIGH
	LED_TRIS=0;
	LED_PIN=0;
/* initialize 7 segment port */
	LED7_SEGPORT = 0x00;	//-- Shut down all led
	LED7_SEGDIR  = 0x00;	//-- Set as Output

	HUNDREDS_7SEG = 1;	//-- Disable all 7Segment
	HUNDREDS_7SEG_TRIS = 0;
	TENS_7SEG = 1;
	TENS_7SEG_TRIS = 0;
	ONES_7SEG = 1;
	ONES_7SEG_TRIS = 0;
	POINT_7SEG = 1;
	POINT_7SEG_TRIS = 0;
	Refresh_Count = 1;
/* initialize timer1 interrupt */
	T1CON = 0x3D;
 	TMR1H = 0xF6;
	TMR1L = 0x3B;
	TMR1ON = 1;		//-- enable timer1
	TMR1IE = 1;		// Enable interrupt on TMR0 overflow
	PEIE = 1;
	GIE = 1;		// Global interrupt enable

 while(1)
 {
  ReadTemp();			//-- Read TEMPERATURE form DS1820
 }
}
//*********** END OF main
static void interrupt
isr(void)			// Here be interrupt function - the name is
				// unimportant.
{
if(TMR1IF) {			// timer interrupt
 TMR1H = 0xFD;
 TMR1L = 0x8E;
 TMR1IF = 0;			// clear the interrupt flag
 Refresh_7Segment();
 }
}
//************************************************************************
//                          ReadTemp()
//
// Read temperature from DS1820
//************************************************************************
void ReadTemp(void)
{
  LED_PIN=0;
  GIE = 0;		// Disble all interrupt
  D_Reset();
  D_Write(0xCC);		//-- SKIP PROM command
  D_Write(0x44);		//-- START CONVERSE command
  GIE = 1;		// Disble all interrupt
  DelayMs(500);                 //-- Big delay before reading
  DelayMs(500);                 //-- Big delay before reading
  DelayMs(500);                 //-- Big delay before reading
  LED_PIN=1;
  GIE = 0;		// Disble all interrupt
  D_Reset();
  D_Write(0xCC);		//-- SKIP PROM command
  D_Write(0xBE);		//-- READ SCRATCH PAD command
  D_Mag=D_Read();		//-- Read magniture
  D_Sign=D_Read();		//-- Read SIGN (FF=-VE, 00=+VE)
  if ((D_Sign == 0) || (D_Sign == 0xFF)) ConvertForDisplay();
  GIE = 1;		// Disble all interrupt
  DelayMs(500);                 //-- Big delay before reading
}
//************************************************************************
//                       ConvertForDisplay
//************************************************************************
void ConvertForDisplay(void)
{
 signed char temp=0;
 static bit point;
 //---- Clear all values to start with -----
 Area.Hundreds=0;
 Area.Tens=0;
 Area.Ones=0;
 Area.Point=0;
 //---- Convert values to Editable Digits ----
 temp = D_Mag;
 point = (D_Mag &= 0x1);
 if(D_Sign == 0xFF)
 {
  temp = ((-temp)/2)+1;
  Area.Hundreds=T_NEGATIVE;
 }
 else
 {
  temp = (temp/2);
  if(temp >99)
  {
   Area.Hundreds=(temp/100);
   temp-=(Area.Hundreds*100);
  }
 }
 if(temp >9)
 {
  Area.Tens=(temp/10);
  temp-=(Area.Tens*10);
 }
 Area.Ones=temp;
 if(Area.Hundreds > 2)Area.Hundreds=0;
 if(Area.Tens > 9)Area.Tens=9;
 if(Area.Ones > 9)Area.Tens=9;
 if(point)Area.Point=5;else Area.Point=0;
 //---- Convert to 7 segment -----
 if(Area.Hundreds == 2) SevenSeg.Hundreds = MINUS;
 else if(Area.Hundreds == 1) SevenSeg.Hundreds = Digits[Area.Hundreds];
 else SevenSeg.Hundreds = LBLANK;
 SevenSeg.Tens = Digits[Area.Tens];
 SevenSeg.Ones = Digits[Area.Ones]+LDOT;
 SevenSeg.Point = Digits[Area.Point];
}
//************************************************************************
//                       Refresh 7 segment
//************************************************************************
void Refresh_7Segment(void)
{
  COMMON_SEGPORT = 0x0;
  switch(Refresh_Count) //-- Current Edit Mode
  {
   case 0:
    LED7_SEGPORT=SevenSeg.Point;
    COMMON_SEGPORT = 0x1;
    break;
   case 1:
    LED7_SEGPORT=SevenSeg.Ones;
    COMMON_SEGPORT = 0x2;
    break;
   case 2:
    LED7_SEGPORT=SevenSeg.Tens;
    COMMON_SEGPORT = 0x4;
    break;
   case 3:
    LED7_SEGPORT=SevenSeg.Hundreds;
    COMMON_SEGPORT = 0x8;
    break;
  }
 Refresh_Count=Refresh_Count+1;
 if (Refresh_Count==4) Refresh_Count=0;
}

//*********** END OF ConvertForDisplay

